@@ -57,6 +57,13 @@ class Adapter(object):
     :param str client_version: The version of the client that created the
                                adapter. This will be used to create the
                                user_agent.
+    :param bool allow_version_hack: Allow keystoneauth to hack up catalog
+                                    URLS to support older schemes.
+                                    (optional, default True)
+    :param str global_request_id: A global_request_id (in the form of
+                                  ``req-$uuid``) that will be passed on all
+                                  requests. Enables cross project request id
+                                  tracking.
     """
 
     client_name = None
@@ -68,7 +75,8 @@ class Adapter(object):
                  version=None, auth=None, user_agent=None,
                  connect_retries=None, logger=None, allow={},
                  additional_headers=None, client_name=None,
-                 client_version=None):
+                 client_version=None, allow_version_hack=None,
+                 global_request_id=None):
         # NOTE(jamielennox): when adding new parameters to adapter please also
         # add them to the adapter call in httpclient.HTTPClient.__init__ as
         # well as to load_adapter_from_argparse below if the argument is
@@ -87,6 +95,9 @@ class Adapter(object):
         self.logger = logger
         self.allow = allow
         self.additional_headers = additional_headers or {}
+        self.allow_version_hack = allow_version_hack
+
+        self.global_request_id = global_request_id
 
         if client_name:
             self.client_name = client_name
@@ -104,6 +115,8 @@ class Adapter(object):
             kwargs.setdefault('region_name', self.region_name)
         if self.version:
             kwargs.setdefault('version', self.version)
+        if self.allow_version_hack is not None:
+            kwargs.setdefault('allow_version_hack', self.allow_version_hack)
 
     def request(self, url, method, **kwargs):
         endpoint_filter = kwargs.setdefault('endpoint_filter', {})
@@ -139,6 +152,10 @@ class Adapter(object):
         for k, v in self.additional_headers.items():
             kwargs.setdefault('headers', {}).setdefault(k, v)
 
+        if self.global_request_id is not None:
+            kwargs.setdefault('headers', {}).setdefault(
+                "X-OpenStack-Request-ID", self.global_request_id)
+
         return self.session.request(url, method, **kwargs)
 
     def get_token(self, auth=None):
@@ -231,6 +248,7 @@ class Adapter(object):
     def delete(self, url, **kwargs):
         return self.request(url, 'DELETE', **kwargs)
 
+    # TODO(efried): Move this to loading.adapter.Adapter
     @classmethod
     def register_argparse_arguments(cls, parser, service_type=None):
         """Attach arguments to a given argparse Parser for Adapters.
@@ -280,6 +298,7 @@ class Adapter(object):
             default=os.environ.get('OS_API_VERSION', None),
             help='Which version of the service API to use')
 
+    # TODO(efried): Move this to loading.adapter.Adapter
     @classmethod
     def register_service_argparse_arguments(cls, parser, service_type):
         """Attach arguments to a given argparse Parser for Adapters.
@@ -381,9 +400,13 @@ class LegacyJsonAdapter(Adapter):
         return resp, body
 
 
+# TODO(efried): Deprecate this in favor of
+#               loading.adapter.register_argparse_arguments
 def register_adapter_argparse_arguments(*args, **kwargs):
     return Adapter.register_argparse_arguments(*args, **kwargs)
 
 
+# TODO(efried): Deprecate this in favor of
+#               loading.adapter.register_service_argparse_arguments
 def register_service_adapter_argparse_arguments(*args, **kwargs):
     return Adapter.register_service_argparse_arguments(*args, **kwargs)
